package JoelCastellanosAnimations;

import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.PixelWriter;
import javafx.scene.paint.Color;

import java.util.Random;

public class DLA extends Animation
{
  private Canvas canvas;
  private GraphicsContext gfx;
  private static final int NUM_POINTS = 300;
  private Point[] point = new Point[NUM_POINTS];

  private static final int[] dx = {1, 0, -1, 0, 1,  1, -1, -1};
  private static final int[] dy = {0, 1, 0, -1, 1, -1,  1, -1};
  private int step;
  private Random random = new Random();

  private int blocksize = 5;
  private int numBlockX = DRAW_WIDTH / blocksize;
  private int numBlockY = DRAW_HEIGHT / blocksize;

  private int[][] grid = new int[numBlockX+2][numBlockY+2];


  private static final Color background = Color.rgb(255, 200, 200);
  private PixelWriter pixelWriter;

  public DLA(Canvas canvas)
  {
    this.canvas = canvas;
    gfx = canvas.getGraphicsContext2D();

    step = 0;

    gfx.setLineWidth(3);

    gfx.setFill(background);
    gfx.fillRect(0, 0, DRAW_WIDTH, DRAW_HEIGHT);

    pixelWriter = canvas.getGraphicsContext2D().getPixelWriter();

    for (int i = 0; i < NUM_POINTS; i++)
    {
      point[i] = new Point(0, 0);
      movePointToBottomEdge(point[i]);

      gfx.setStroke(Color.BLUE);

      renderBlock(point[i].x,point[i].y, Color.BLUE);
    }

    crystilize(numBlockX / 2, numBlockY / 2);
    //printGridAndPoints(grid, point);
  }


  public void handle(long now)
  {
    step++;
    //if (step % 100 != 0) return;

    //System.out.println(step + ": " + point.x + ", " + point.y);

    for (int i = 0; i < NUM_POINTS; i++)
    {
      renderBlock(point[i].x,point[i].y, background);

      double r = random.nextDouble();
      if (r < 0.25)
      {
        if (point[i].y > 1) { point[i].y -= 1; }
        else { movePointToBottomEdge(point[i]); }
      }
      else if (r < 0.5)
      {
        if (point[i].y < numBlockY) point[i].y += 1;
      }
      else if (r < 0.75)
      {
        if (point[i].x < numBlockX) { point[i].x += 1; }
        else { movePointToBottomEdge(point[i]); }
      }
      else
      {
        if (point[i].x > 1) { point[i].x -= 1; }
        else { movePointToBottomEdge(point[i]); }
      }


      for (int k=0; k<dx.length; k++)
      {
        int xx = point[i].x + dx[k];
        int yy = point[i].y + dy[k];
        if (grid[xx][yy] > 0)
        {
          crystilize(point[i].x, point[i].y);
          movePointToBottomEdge(point[i]);
        }
      }


      //pixelWriter.setColor(point.x, point.y,Color.BLUE);
      renderBlock(point[i].x,point[i].y, Color.BROWN);
    }

    //printGridAndPoints(grid, point);
  }

  //===================================================================================
  //===================================================================================
  private void movePointToBottomEdge(Point p)
  {
    p.x = 1 + (int) (random.nextDouble() * numBlockX);
    p.y = numBlockY - 1;
  }


  //===================================================================================
  //===================================================================================
  private void crystilize(int x, int y)
  {
    grid[x][y] = 1;
    renderBlock(x,y, Color.GREEN);
    }



  //===================================================================================
  //===================================================================================
  private void renderBlock(int x, int y, Color c)
  {
    gfx.setFill(c);
    gfx.fillRect((x-1) * blocksize, (y-1) * blocksize, blocksize, blocksize);
  }


  //===================================================================================
  //===================================================================================
  private static void printGridAndPoints(int[][] myGrid, Point[] myPointList)
  {
    //Print points and grid together.
    for (int y = 1; y < myGrid[0].length-1; y++)
    {
      for (int x = 1; x < myGrid.length-1; x++)
      {

        //Loop through all the points to see if one is in this grid space.
        boolean spaceHasPoint = false;
        for (int p = 0; p < myPointList.length; p++)
        {
          if (myPointList[p].x == x && myPointList[p].y == y)
          {
            spaceHasPoint = true;
            System.out.print('o');
          }
        }


        //Only if a point was not found in this grid space, then print either
        //  a period if 0 is in the space.  Otherwise print an X.
        if (spaceHasPoint == false)
        {
          if (myGrid[x][y] == 0) { System.out.print('.'); }
          else { System.out.print('X'); }
        }
      }
      System.out.println(); //at the end of the x loop, we are done with a row.
    }
    System.out.println("\n");
  }

  //===================================================================================
  //   Point() is a simple data class with public fields and no methods.
  //===================================================================================
  class Point
  {
    public int x, y;

    public Point(int x, int y)
    {
      this.x = x;
      this.y = y;
    }
  }

}
